#!/bin/bash

# scaler.sh
# Part of the Zerogame Project emulation suite for PS3 Linux
# By Jason Carter (jay@zerojay.com)

# Based heavily on billb's original mame-scaler.sh script.

source /home/zerogame/.zerogame/zerogamerc

# I feel a bit of an explanation is needed for this script since it's somewhat complex compared 
# to billb's original and it may not be as straightforward to understand as it should be.
# On the PS3, we have no access to hardware graphics acceleration. As a workaround for this,
# a program was created by miya, a Japanese PS3 Linux hacker, which allowed us to use the
# normally unused SPE cell units to upscale our emulator output to higher resolutions. All the
# emulators installed as part of Zerogame are configured to output their video at the original
# resolution the games used. When a game is selected from the Wahcade game list, this script
# decides, through various means, what scaling should be used without screwing up the aspect
# ratio and without lopping off huge parts of the image. It does the best that it can, however
# the scaler program used has some limitations in that it can only do certain scale sizes, so
# do not always expect every pixel on your screen to be used at all times.

# Command line input:
# scaler.sh [console name] [scaling type] [game name/path to game]

# [console name]
# This is the shortened Zerogame standardized console name (gbc instead of Gameboy Color, 
# for example). This allows us to special case emulators and consoles which use the same
# standard resolution and to dynamically decide what to do on a per-game basis for MAME.
# This parameter is required.

# [scaling type]
# This parameter allows the user to decide what type of scaling they would like to use; standard 
# scaling or smoothed scaling. Smoothed scaling means that the scaler will look at each pixel and
# will change the image to appear less jagged and blocky. If this parameter is omitted, standard
# scaling will be used. If fscaler is provided as the parameter passed, the scaler script will
# attempt to use the scaleXx method of smoothing. If sscaler is provided as the parameter passed,
# sai2x will be used instead.
# This parameter is optional.

# [game name/path to game]
# This parameter tells the script which game was selected by the user in Wahcade and launches
# it with the proper emulator.
# This parameter is required.

# First, determine the current display dimensions X is using as provided by xdpyinfo.
DISPLAY_DIMENSIONS=`xdpyinfo | grep dimensions | awk '{ print $2 }'`
DISPLAY_HEIGHT=`echo $DISPLAY_DIMENSIONS | cut -f2 -d x`
DISPLAY_WIDTH=`echo $DISPLAY_DIMENSIONS | cut -f1 -d x`
echo "X11 display width: $DISPLAY_WIDTH"
echo "X11 display height: $DISPLAY_HEIGHT"

# Rip out the last parameter passed and use it as the game to be launched.
eval GAMENAME=\$$#

# Determine the standard required resolution for the console/game being run.
# Most consoles have standard resolutions that their games run under, however
# because MAME emulates so many different hardware setups, it's impossible
# to select one standard resolution that works for all games. Instead, we use
# the MAME game info XML file to look up the game we want to play and grab
# the game's resolution from there and run with it.

# If the game we're running is supposed to be run by MAME...
if [ "$1" = "mame" ]; then
  EMU_BIN="mame"
  # Setting defaults that should run any games w/o valid resolution info as 
  # unscaled.
  GAME_WIDTH="800"
  GAME_HEIGHT="800"
  ROTATE="0"
  GAME_XML=`$EMU_BIN -lx $GAMENAME | grep "<display " | grep -v ATTLIST | column -t`
  # ...grab the game's resolution directly from the MAME game info XML output.
  for i in $GAME_XML; do
    # Is the game a vertical or horizontally oriented game?
    if [ `echo $i | cut -f1 -d =` == 'rotate' ]; then
      ROTATE=`echo $i | cut -f2 -d = | cut -f2 -d \"`
    fi
    if [ `echo $i | cut -f1 -d =` == 'width' ]; then
      GAME_WIDTH=`echo $i | cut -f2 -d = | cut -f2 -d \"`
    fi
    if [ `echo $i | cut -f1 -d =` == 'height' ]; then
      GAME_HEIGHT=`echo $i | cut -f2 -d = | cut -f2 -d \"`
    fi
    # If display type is a vector game, set static resolution. Most vector 
    # games seem to use 640x480 w/ no scaling in MAME, so arbitrarily using 
    # 464x464 to account for rotated games and so 720p will be scale by 1.5x 
    # and 1080i/p will scale by 2x.
    # Modification by xnonsuchx (steve@nonsuch.us) on 20100625
    if [ `echo $i | cut -f1 -d =` == 'type' ]; then
      GAME_DISPLAY_TYPE=`echo $i | cut -f2 -d = | cut -f2 -d \"`
    fi
    if [ "$GAME_DISPLAY_TYPE" = "vector" ]; then
      GAME_WIDTH="464"
      GAME_HEIGHT="464"
    fi
  done

  # Frogger appears to be a strange exception. The reported MAME resolution is 
  # rotated 768x224 (7:24 aspect ratio), so we need to adjust that to a 3:4 aspect 
  # ratio instead. Video Options also need to be set to 'Standard (3:4)' so MAME 
  # .cfg files are also needed.
  if [ "$GAMENAME" = "frogger" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "froggers" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "frogf" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "frogg" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "froggers1" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "froggers2" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "froggermc" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  # These are the newer MAME version names for froggers1 & froggers2, which didn't 
  # follow the 8-character ROMSET naming convention (both need to be in the script 
  # since we don't know for sure which version of MAME will be running).
  if [ "$GAMENAME" = "frogseg1" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "frogseg2" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi

  # Ditto for Galaxian and clones.
  if [ "$GAMENAME" = "galaxian" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galap1" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galap4" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galapx" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galaxbsf" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galaxiaj" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galmidw" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galmidwo" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galtaito" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "galturbo" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "moonal2" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "moonal2b" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "moonaln" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "starfght" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "superg" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "swarm" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "tst_galx" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "zerotime" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi

  # Ditto for Super Cobra and clones.
  if [ "$GAMENAME" = "scobra" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "scobras" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "scobrase" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "scobrab" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "suprheli" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi

  # Ditto for Moon Cresta and clones.
  if [ "$GAMENAME" = "mooncrst" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "eagle" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "eagle2" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "eagle3" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "fantazia" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncmw" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrgx" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrs2" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrs3" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrsa" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrsb" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrsg" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "mooncrsu" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "smooncrs" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "spcdrag" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "spcdraga" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "spctbird" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi

  # Ditto for misc. others.
  if [ "$GAMENAME" = "moonwar" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi
  if [ "$GAMENAME" = "moonwara" ]; then
     GAME_HEIGHT="224"
     GAME_WIDTH="296"
  fi

  # And prevent some vector games from mis-scaling.
  if [ "$GAMENAME" = "quantum" ]; then
     GAME_HEIGHT="640"
     GAME_WIDTH="640"
  fi
  if [ "$GAMENAME" = "quantum1" ]; then
     GAME_HEIGHT="640"
     GAME_WIDTH="640"
  fi
  if [ "$GAMENAME" = "quantump" ]; then
     GAME_HEIGHT="640"
     GAME_WIDTH="640"
  fi
  if [ "$GAMENAME" = "tacscan" ]; then
     GAME_HEIGHT="640"
     GAME_WIDTH="640"
  fi

  # And some misc. games w/ odd aspects.
  if [ "$GAMENAME" = "blstroid" ]; then
     GAME_HEIGHT="240"
     GAME_WIDTH="320"
  fi
  if [ "$GAMENAME" = "darius" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="864"
  fi
  if [ "$GAMENAME" = "dariusj" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="864"
  fi
  if [ "$GAMENAME" = "dariuse" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="864"
  fi
  if [ "$GAMENAME" = "dariuso" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="864"
  fi
  if [ "$GAMENAME" = "darius2" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="864"
  fi
  if [ "$GAMENAME" = "darius2d" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="640"
  fi
  if [ "$GAMENAME" = "darius2do" ]; then
     GAME_HEIGHT="232"
     GAME_WIDTH="640"
  fi


  # If the game is vertical, swap the reported width and height.
  # Note: Some games are reported as rotated 90 degrees and others are reported as rotated 270 degrees.
  # This fixes a major issue where many vertical games weren't being properly scaled without clipping.
  if [ $ROTATE == 90 ] || [ $ROTATE == 270 ]; then
    TEMP_HEIGHT=$GAME_HEIGHT
    GAME_HEIGHT=$GAME_WIDTH
    GAME_WIDTH=$TEMP_HEIGHT
  fi

  # A major issue with running MAME on the PS3 is speed. We have zero graphical acceleration, so some
  # stuff just absolutely CRAWLS. MAME often slows down as it gets older because emulation gets more
  # accurate and MAMEDev generally assumes that people will upgrade along with them. That's not really
  # a possibility for us on the PS3, so we're going to have to find other ways to speed things up.
  # One of the possibilities that the PS3 Linux community have explored is using older versions of MAME.
  # Games that use the DCS sound system slowed down a ton at some point because DCS's accuracy jumped
  # up a ton. The problem for us is that most of them are so slow as to be useless and unplayable on
  # the PS3. So our workaround is to install an older version of XMAME and to switch over to that build
  # when we see certain games being run. This ends up working out quite well.
  # Pros: Playable, faster speed, not really noticable to the user.
  # Cons: Added complexity in Zerogame, some games run without sound, some ROM sets have changed and need
  #       to be updated.
  #
  # As a huge fan of NBA Hangtime... I'm sorry folks, but the pros win out. Hopefully we can use a build
  # that will restore sound. For now, this will have to do.
  if [ $USEXMAMEHACKS = "1" ]; then
    if [ "$GAMENAME" = "openice" ]; then
       # Open Ice runs but without sound.
       echo "Detected NHL Open Ice... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "sftm" ]; then
       # Street Fighter: The Movie runs well, if a slight bit choppy. Still very playable.
       # Also: http://alan-noon.blogspot.com/2007/08/follow-up-street-fighter-movie.html
       # Also: http://www.alan-noon.com/misc/SFTM/SFTM_Broke_My_Heart.pdf
       # Some very interesting stuff. Check it out. And by the way, the console versions
       # were different and *SO* much better.
       echo "Detected Street Fighter: The Movie... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "sftmj" ]; then
       echo "Detected Street Fighter: The Movie... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "sftm110" ]; then
       echo "Detected Street Fighter: The Movie... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "sftm110" ]; then
       echo "Detected Street Fighter: The Movie... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "wwfmania" ]; then
       # WWF Wrestlemania runs but without sound.
       # SECOND WIND!
       echo "Detected WWF Wrestlemania... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "wwfmaniab" ]; then
       # WWF Wrestlemania runs but without sound.
       # Switch to "wwfmania" as "wwfmaniab" didn't exist
       # in older versions of MAME.
       GAMENAME="wwfmania"
       echo "Detected WWF Wrestlemania... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbahangt" ]; then
       # NBA Hangtime works but without sound.
       # I'm in the zone!
       echo "Detected NBA Hangtime... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbamht1" ]; then
       # NBA Maximum Hangtime works but without sound.
	   # This version of the game got renamed nbamht1 back around v0.81 of MAME.
	   # Using the older name so xmame does the right thing.
	   GAMENAME="nbamaxht"
       echo "Detected NBA Maximum Hangtime... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbamht" ]; then
       # NBA Maximum Hangtime works but without sound.
       echo "Detected NBA Maximum Hangtime... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbajam" ]; then
       # NBA JAM works... with sound!
       # BOOMSHAKALAKA!
       echo "Detected NBA JAM... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbajamr2" ]; then
       # Clone of nbajam.
       echo "Detected NBA JAM... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "nbajamt1" ]; then
       # NBA JAM: TE works with sound!
       echo "Detected NBA JAM: TE... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r14" ]; then
       # Mortal Kombat 2 runs but without sound.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2" ]; then
       # See above.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r21" ]; then
       # See above.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r32" ]; then
       # See above.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r42" ]; then
       # See above.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r91" ]; then
       # See above.
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r30" ]; then
       # We've got an interesting little issue here.
       # See, this revision of Mortal Kombat 2 didn't exist back in the days
       # of XMAME. Attempting to run xmame with "mk2r30" as the game results
       # in xmame running, of all things, Ms. Pac Attack!
       # To avoid this issue, and to make sure the proper game is launched
       # (even if it's not the same revision), we'll change the game name to
       # "mk2" which should be fine with xmame.
       GAMENAME="mk2"
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2r31e" ]; then
       # See note on "mk2r30" above. Same deal.
       GAMENAME="mk2"
       echo "Detected Mortal Kombat 2... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mk2chal" ]; then
       # See mk2.
       echo "Detected Mortal Kombat 2 Challenger Hack... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mkprot8" ]; then
       GAMENAME="mk"
       echo "Detected Mortal Kombat... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mkyturbo" ]; then
       GAMENAME="mk2"
       echo "Detected Mortal Kombat... using xmame."
       EMU_BIN="xmame.SDL"
    fi
    if [ "$GAMENAME" = "mktturbo" ]; then
       GAMENAME="mkprot8"
       echo "Detected Mortal Kombat... using xmame."
       EMU_BIN="xmame.SDL"
    fi
  fi
fi


# If the game isn't a MAME game, we can more or less assume what resolution the game should be running
# at based on the most common resolutions used by the games themselves. Most of this data comes directly
# from the games themselves or from Wikipedia.
# We also use the console name to determine which emulator we should use to run the game.

if [ "$1" = "c64" ]; then
  EMU_BIN="x64"
  # The following are custom configs for use w/ VICE 2.1
  #if [ $DISPLAY_HEIGHT = "644" ]; then
  #EMU_BIN="x64 -config /home/zerogame/.vice/VICE2.1/vicerc-720pb"
  #fi
  #if [ $DISPLAY_HEIGHT = "720" ]; then
  #EMU_BIN="x64 -config /home/zerogame/.vice/VICE2.1/vicerc-720p"
  #fi
  #if [ $DISPLAY_HEIGHT = "964" ]; then
  #EMU_BIN="x64 -config /home/zerogame/.vice/VICE2.1/vicerc-1080pb"
  #fi
  #if [ $DISPLAY_HEIGHT = "1080" ]; then
  #EMU_BIN="x64 -config /home/zerogame/.vice/VICE2.1/vicerc-1080p"
  #fi
  GAME_WIDTH="384"
  GAME_HEIGHT="270"
  # Check if scaling should be done w/o C64 display overscan area 
  # per Tools script for the user to select.
  if [ -f /home/zerogame/.vice/no_overscan ]; then
    GAME_WIDTH="320"
    GAME_HEIGHT="200"
  fi
fi
 # Add an exception for C64 cartridge files needing a specific 
 # command line argument added to load them.
if [ "$EMU_BIN" = "x64" ]; then
  C64GAMEEXT=`echo "${GAMENAME#*.}"`
  if [ `echo "$C64GAMEEXT" | tr [:upper:] [:lower:]` = "crt.zip" ]; then
  EMU_BIN="x64 -cartcrt"
  fi
fi

if [ "$1" = "fds" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="256"
  GAME_HEIGHT="224"
fi

if [ "$1" = "sms" ]; then
   EMU_BIN="mednafen"
   GAME_WIDTH="256"
   GAME_HEIGHT="240"
fi

if [ "$1" = "gamegear" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="144"
fi

if [ "$1" = "gb" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="144"
fi

if [ "$1" = "gba" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="240"
  GAME_HEIGHT="160"
fi

if [ "$1" = "gbc" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="144"
fi

if [ "$1" = "genesis" ]; then
  EMU_BIN="xe"
  # Figure out # of active controllers to use correct config.
  GENCTRLRS=`ls /dev/input/js* | wc -l`
  GAME_WIDTH="320"
  GAME_HEIGHT="224"
  # Deal with possibly missing bios file keeping Genesis games from running if the SegaCD bios isn't around. Annoying.
  if [ -f /home/zerogame/.xe/bios/megacd-u.rom ]; then
    cp /home/zerogame/.zerogame/configs/xe/rc/xerc /home/zerogame/.xe/rc/xerc
    if [ $GENCTRLRS = "2" ]; then
      cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_2 /home/zerogame/.xe/rc/xerc
    fi
    if [ $GENCTRLRS = "3" ]; then
      cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_3 /home/zerogame/.xe/rc/xerc
    fi
    if [ $GENCTRLRS = "4" ]; then
      cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_4 /home/zerogame/.xe/rc/xerc
    fi
    cp /home/zerogame/.zerogame/configs/xe/rc/megadriverc /home/zerogame/.xe/rc/megadriverc
  else
    cp /home/zerogame/.zerogame/configs/xe-nocd/rc/xerc /home/zerogame/.xe/rc/xerc
    if [ $GENCTRLRS = "2" ]; then
      cp /home/zerogame/.zerogame/configs/xe-nocd/rcalt/xerc_2 /home/zerogame/.xe/rc/xerc
    fi
    if [ $GENCTRLRS = "3" ]; then
      cp /home/zerogame/.zerogame/configs/xe-nocd/rcalt/xerc_3 /home/zerogame/.xe/rc/xerc
    fi
    if [ $GENCTRLRS = "4" ]; then
      cp /home/zerogame/.zerogame/configs/xe-nocd/rcalt/xerc_4 /home/zerogame/.xe/rc/xerc
    fi
    cp /home/zerogame/.zerogame/configs/xe-nocd/rc/megadriverc /home/zerogame/.xe/rc/megadriverc
  fi
fi

if [ "$1" = "nes" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="256"
  GAME_HEIGHT="224"
fi

if [ "$1" = "ngpocket" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="152"
fi 

if [ "$1" = "ngpocketcolor" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="152"
fi 

if [ "$1" = "segacd" ]; then
  EMU_BIN="xe"
  # Figure out # of active controllers to use correct config.
  GENCTRLRS=`ls /dev/input/js* | wc -l`
  GAME_WIDTH="320"
  GAME_HEIGHT="224"
  cp /home/zerogame/.zerogame/configs/xe/rc/xerc /home/zerogame/.xe/rc/xerc
  if [ $GENCTRLRS = "2" ]; then
    cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_2 /home/zerogame/.xe/rc/xerc
  fi
  if [ $GENCTRLRS = "3" ]; then
    cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_3 /home/zerogame/.xe/rc/xerc
  fi
  if [ $GENCTRLRS = "4" ]; then
    cp /home/zerogame/.zerogame/configs/xe/rcalt/xerc_4 /home/zerogame/.xe/rc/xerc
  fi
  cp /home/zerogame/.zerogame/configs/xe/rc/megadriverc /home/zerogame/.xe/rc/megadriverc
fi

if [ "$1" = "tg16" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="256"
  GAME_HEIGHT="240"
fi

if [ "$1" = "intv" ]; then
  EMU_BIN="/home/zerogame/.zerogame/wrappers/jzintv-launch.sh"
  GAME_WIDTH="320"
  GAME_HEIGHT="200"
fi

if [ "$1" = "snes" ]; then
  EMU_BIN="snes9x-gtk"
  GAME_WIDTH="256"
  GAME_HEIGHT="224"
  cp ~/.snes9x/snes9x-orig.xml ~/.snes9x/snes9x.xml
  if ( [[ "$GAMENAME" =~ "Secret of Mana" ]] ); then
   GAMEEXCEPTION=1
   MANAHACK=1
   echo "Secret of Mana workaround."
  fi
  if ( [[ "$GAMENAME" =~ "Seiken Densetsu 2" ]] ); then
   GAMEEXCEPTION=1
   MANAHACK=1
   echo "Secret of Mana workaround."
  fi
fi

# The Atari Lynx, Bandai WonderSwan and Bandai WonderSwan Color are exceptions for one
# main reason: their consoles can often be played in a rotated style. To avoid trying to figure
# out which way we need to scale, we lie and say that the screen size for these consoles is square.
# This allows us to scale the image without needing to care about what orientation they are
# currently running in, which would have been impossible to deal with without special casing
# each and every little game... too much work for me. This only works because the resolution these
# handhelds run at aren't that far off from being square anyways.

if [ "$1" = "lynx" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="160"
  GAME_HEIGHT="160"
fi

if [ "$1" = "ws" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="224"
  GAME_HEIGHT="224"
fi

if [ "$1" = "wscolor" ]; then
  EMU_BIN="mednafen"
  GAME_WIDTH="224"
  GAME_HEIGHT="224"
fi

# Output the screen's width and height for debugging purposes.

echo "Game's screen width: $GAME_WIDTH"
echo "Game's screen height: $GAME_HEIGHT"

# Calculate the best scaling factor for the game in question by comparing the game's width
# and height to the size of the screen.

# Because of the way the scaler program worked up until now, we weren't previously
# able to scale to anything other than full integer factors (2x, 3x, etc...). This meant
# that we often were stuck with smaller scaling sizes than we'd like. The latest version
# of fscaler does allow you to use two passes on the image, with the second pass allowing
# downscaling by half. So to get some scaling factors such as 1.5x and 2.5x, you must be able
# to scale up to 3x and 5x first and then use the "half" scaling on the second pass.

# The SCALING_FACTOR variables here are calculated best scaling factors to the nearest
# full integer number. The SUBSCALING variables are used to figure out if we can go an
# extra 0.5x or not.

#By dividing the display size by the game size, we can find out the best scaling size to use.
SCALING_FACTOR_X=$(($DISPLAY_WIDTH/$GAME_WIDTH))

# However, when we're dealing with 0.5x magnifications, it doesn't really work. What we do
# instead is divide the same two numbers and see if there is a remainder. If the remainder is
# larger than half the game size, we know we can go with a non-integer scaling mode instead.

SUBSCALING_X=$(($DISPLAY_WIDTH%$GAME_WIDTH*2))
echo "Subscaling Factor X: $SUBSCALING_X"

# We could have set SUBSCALING to any number other than zero, really... but picking 5
# means it's easier to output the actual scaling factor for debugging.
if [ $SUBSCALING_X -gt $GAME_WIDTH ]; then
   SUBSCALING_X=5
else
   SUBSCALING_X=0
fi

# We've figured out the best scaling factor horizontally, now let's do the same calculations
# on the Y axis.

SCALING_FACTOR_Y=$(($DISPLAY_HEIGHT/$GAME_HEIGHT))
SUBSCALING_Y=$(($DISPLAY_HEIGHT%$GAME_HEIGHT*2))
echo "Subscaling Factor Y: $SUBSCALING_Y"
if [ $SUBSCALING_Y -gt $GAME_HEIGHT ]; then
   SUBSCALING_Y=5
else
   SUBSCALING_Y=0
fi

echo "Optimal integer X scaling factor: $SCALING_FACTOR_X.$SUBSCALING_X"
echo "Optimal integer Y scaling factor: $SCALING_FACTOR_Y.$SUBSCALING_Y"



# Now that we've calculated the best scaling factors horizontally and vertically, we
# compare the two scaling factors and choose the smaller of the two. This is done
# because if we used the larger one instead, at least some part of the screen would
# be cut off. Chosing the smaller of the two ensures that we should always be able
# to see everything going on.

if [ "$SCALING_FACTOR_X" -le "$SCALING_FACTOR_Y" ]; then
   SCALING_FACTOR_T=`echo $SCALING_FACTOR_X`
   if [ "$SCALING_FACTOR_X" -lt "$SCALING_FACTOR_Y" ]; then
      SUBSCALING_T=`echo $SUBSCALING_X`
   else
      if [ "$SUBSCALING_X" -le "$SUBSCALING_Y" ]; then
         SUBSCALING_T=`echo $SUBSCALING_X`
      else
         SUBSCALING_T=`echo $SUBSCALING_Y`
      fi
   fi
else
   SCALING_FACTOR_T=`echo $SCALING_FACTOR_Y`
   SUBSCALING_T=`echo $SUBSCALING_Y`
fi

# Let's throw in some exceptions that don't lose much in the upscaling.
if [ $USESCALINGEXCEPTIONS = "1" ]; then
 if [ "$1" = "mame" ]; then
  if [ "$DISPLAY_HEIGHT" = "720" ]; then
   if [ "$GAMENAME" = "openice" ]; then
     echo "NHL Open Ice scaling exception: 3x on 720p."
     SCALING_FACTOR_T="3"
     SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "sftm" ]; then
    echo "Street Fighter: The Movie scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "sftmj" ]; then
    echo "Street Fighter: The Movie scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "sftm111" ]; then
    echo "Street Fighter: The Movie scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "sftm110" ]; then
    echo "Street Fighter: The Movie scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "wwfmania" ]; then
    echo "WWF Wrestlemania scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "nbahangt" ]; then
    echo "NBA Hangtime scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "darius" ]; then
    echo "Darius scaling exception: 1.5x on 720p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "dariusj" ]; then
    echo "Darius scaling exception: 1.5x on 720p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "dariuso" ]; then
    echo "Darius scaling exception: 1.5x on 720p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "dariuse" ]; then
    echo "Darius scaling exception: 1.5x on 720p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "darius2" ]; then
    echo "Darius scaling exception: 1.5x on 720p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "punchout" ]; then
    echo "Punchout scaling exception: 2.5x on 720p."
    SCALING_FACTOR_T="2"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "spnchout" ]; then
    echo "Super Punchout scaling exception: 2.5x on 720p."
    SCALING_FACTOR_T="2"
    SUBSCALING_T="5"
   fi
   if [ "$GAMENAME" = "mk2" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2r14" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2r21" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2r32" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2r42" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2r91" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk2chal" ]; then
    echo "Mortal Kombat 2 scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mk" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkprot8" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkprot9" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mka1" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkla2" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkla3" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkla4" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "mkr4" ]; then
    echo "Mortal Kombat scaling exception: 3x on 720p."
    SCALING_FACTOR_T="3"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "liberatr" ]; then
    echo "Liberator scaling exception: 4x on 720p."
    SCALING_FACTOR_T="4"
    SUBSCALING_T="0"
   fi
   #if [ "$GAME_HEIGHT" -ge "720" ] && [ "$GAME_HEIGHT" -le "780" ]; then
   # echo "'Slightly too tall' scaling exception: 1x on 720p."
   # SCALING_FACTOR_T="1"
   # SUBSCALING_T="0"
   #fi
  fi
#  if [ "$DISPLAY_HEIGHT" = "1080" ]; then
#   if [ "$GAMENAME" = "darius" ]; then
#    echo "Darius scaling exception: 2x on 1080p."
#    SCALING_FACTOR_T="2"
#    SUBSCALING_T="0"
#   fi
#   if [ "$GAMENAME" = "dariusj" ]; then
#    echo "Darius scaling exception: 2x on 1080p."
#    SCALING_FACTOR_T="2"
#    SUBSCALING_T="0"
#   fi
#  fi
  if [ "$DISPLAY_HEIGHT" = "480" ]; then
   if [ "$GAMENAME" = "darius" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 480p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariusj" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 480p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariuse" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 480p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariuso" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 480p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "darius2" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius II scaling exception: 1x on 480p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
  fi
  if [ "$DISPLAY_HEIGHT" = "576" ]; then
   if [ "$GAMENAME" = "darius" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 576p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariusj" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 576p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariuse" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 576p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "dariuso" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius scaling exception: 1x on 576p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
   if [ "$GAMENAME" = "darius2" ]; then
    echo "Game width is actually 864...game will be cutoff."
    echo "Darius II scaling exception: 1x on 576p."
    SCALING_FACTOR_T="1"
    SUBSCALING_T="0"
   fi
  fi
 fi
 # Add an exception for games w/ undetermined resolution and 
 # launch unscaled (GAMEEXCEPTION entry likely needed).
 if [ "$GAME_HEIGHT" = "unknown" ]; then
  echo "Undetected resolution...no scaling used."
  SCALING_FACTOR_T="1"
  SUBSCALING_T="0"
 fi
 # Add an exception to run any 0.5x optimal scaling factor to run unscaled instead.
 if [ "$SCALING_FACTOR_T.$SUBSCALING_T" = "0.5" ]; then
  echo "Detected resolution too large for display...launching unscaled."
  SCALING_FACTOR_T="1"
  SUBSCALING_T="0"
 fi
fi

echo "Scaling game to $SCALING_FACTOR_T.$SUBSCALING_T"

# Now that we've decided the best scaling factor to use, let's run the emulator using
# the scaling type and scaling factor selected by the user. In some cases, certain
# scaling types and factors are not available due to how the scaler program works.

# As usual, MAME is somewhat of an exception because of required command line
# parameters. (Maybe these two are actually configurable in mame.ini, which
# would greatly simplify this part of the script... but I'm lazy. :)

# We run the scaler program and try to match up with what the user requested as
# best we can. The emulator being run is decided by the console being run, dealt
# with earlier in the script. Perhaps I'll add some more to handle more rarely used
# scaling factors, depending on what I see when I use 1080p.


if [ ! $GAMEEXCEPTION ]; then 
if [ "$2" = "fscaler" ]; then
  if [ "$SCALING_FACTOR_T" = 1 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f scale3x -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
      $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 2 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f near5 -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
      fscaler -f scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 3 ]; then
    fscaler -f scale3x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 4 ]; then
    fscaler -f scale2x -s scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 5 ]; then
    fscaler -f near5 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 6 ]; then
    fscaler -f scale3x -s scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 7 ]; then
    fscaler -f scale3x -s scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 8 ]; then
   fscaler -f near4 -s scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  killall fscaler
  exit
fi
if [ "$2" = "sscaler" ]; then
  if [ "$SCALING_FACTOR_T" = 1 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f scale3x -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
      $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 2 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f near5 -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
      fscaler -f s2xsai -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 3 ]; then
    fscaler -f scale3x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 4 ]; then
    fscaler -f s2xsai -s scale2x -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 5 ]; then
    fscaler -f near5 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 6 ]; then
    fscaler -f scale3x -s s2xsai -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 7 ]; then
    fscaler -f scale3x -s s2xsai -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 8 ]; then
    fscaler -f near4 -s s2xsai -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  killall fscaler
  exit
else
  if [ "$SCALING_FACTOR_T" = 1 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f near3 -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
    $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 2 ]; then
    if [ "$SUBSCALING_T" = 5 ]; then
      fscaler -f near5 -s half -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    else
      fscaler -f near2 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
    fi
  fi
  if [ "$SCALING_FACTOR_T" = 3 ]; then
    fscaler -f near3 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 4 ]; then
    fscaler -f near4 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 5 ]; then
    fscaler -f near5 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 6 ]; then
    fscaler -f near3 -s near2 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 7 ]; then
    fscaler -f near3 -s near2 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  if [ "$SCALING_FACTOR_T" = 8 ]; then
    fscaler -f near4 -s near2 -b 0 -t 999999 & $EMU_BIN "$GAMENAME"
  fi
  # Once the emulator is done running, we kill off the scaler program.
  killall fscaler
fi
else
  if [ $MANAHACK ]; then
   cp ~/.snes9x/snes9x-mana.xml ~/.snes9x/snes9x.xml
   $EMU_BIN "$GAMENAME"
   cp ~/.snes9x/snes9x-orig.xml ~/.snes9x/snes9x.xml
  fi
fi

# To allow use to view the latest output from the last emulator run,
# and because running the viewing script erases the actual log,
# move the log to a place where it won't be screwed with before
# viewing.

cp /home/zerogame/.wahcade/emulator.log /home/zerogame/.zerogame/emulator.log

# Thank you, and good night everybody!
# *crowd cheers*
