#!/bin/bash

# zerogame.sh
# Part of the Zerogame Project emulation suite for PS3
# By Jason Carter (jay@zerojay.com)

# This script is essentially just a wrapper for running Wahcade. I wrote it so that we don't 
#have to deal with different resolutions the user may decide to use. Wahcade doesn't care
#what resolution you are running at and does not adjust to use themes that fit that resolution, 
#so that is part of what this script does.

# The script assumes that you have a version of your Wahcade theme for each resolution the
# user could be running at. At the moment, that means 720p full screen and bordered and 1080p
# full screen and bordered.

# Since Wahcade may not have the proper theme set for the current resolution, we essentially
# use this script to check the resolution the user is currently running under and edit the Wahcade 
#configuration so that the proper resolution of the same theme is used before launching Wahcade.
# This gives the user the impression that Wahcade automatically adjusts to the user's choice of 
# resolution.

# There are a few other things this script does to make things a little nicer for the end user :

# It automatically will check to see if the MAME game list has been created and if not, creates
# it using a script to build it manually instead of using Wahcade's built-in method which leaves
# users thinking that Wahcade's crashed.

# It also launches Gizmo Daemon, a program that allows you to take input from many sources
# and act upon it in many many ways. We're using it to take input from the Sixaxis/DualShock
# and translate the PS button press into quitting back to the PS3 GameOS. Also, we are using
# it to convert button presses into keyboard input for Wahcade itself since it appears to have
# some strange bugs with joystick input which aren't fixed yet. Gizmo also allows us to turn on 
# our PS3 controllers and not require restarting Wahcade before they are recognized. (Emulators
# are still a different story though.

# The actual theme that we want to use comes from a variable stored
# in this file here.
source /home/zerogame/.zerogame/zerogamerc

if [ $CHECKZEROGAMEUPDATES -eq "1" ]; then
   xterm -e /home/zerogame/.zerogame/tools/updatezerogame.sh
fi

# One thing I noticed is that there are few things nastier than having two copies of gizmod
# running at the same time. CPU use skyrockets and things start getting hard to use. I'm taking
# no chances and forcing any open gizmod processes to die before starting a new one.
killall -9 gizmod

# We're going to use one of the little applications that come with PS3 Linux distributions
# to determine the currently running resolution . Once we know the resolution, we then 
# search through the Wahcade configuration file for the line which sets the theme to use.
# We write in the proper theme name and append the resolution to the end of it. For simplcity,
# all themes that Zerogame will support MUST be named with the resolution at the end.

if [ $ZEROGAMEDYNAMICRES -eq "1" ]; then
  VIDEOMODE=`/usr/sbin/ps3-video-mode`

  # If the user is running in 720p...
  if [ $VIDEOMODE -eq "131" ]; then
   echo "720p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   # ...we set the theme to the 720p version. Simple, eh?
   echo "layout                    $WAHCADETHEME-720p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
 
  fi
  if [ $VIDEOMODE -eq "132" ]; then
   echo "1080i fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi

  if [ $VIDEOMODE -eq "133" ]; then
   echo "1080p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080p" >> /home/zerogame/.wahcade/wahcade.tmp 
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "136" ]; then
   echo "720p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-720p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "137" ]; then
   echo "1080i fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "138" ]; then
   echo "1080p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "8" ]; then
   echo "720p bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-720pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "6" ]; then
   echo "576i fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-576p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "7" ]; then
   echo "576p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-576p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "9" ]; then
   echo "1080i bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "10" ]; then
   echo "1080p bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "5" ]; then
   echo "1080p bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "4" ]; then
   echo "1080i bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-1080pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "3" ]; then
   echo "720p bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-720pb" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "2" ]; then
   echo "480p bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-480p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "1" ]; then
   echo "480i bordered!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-480p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "130" ]; then
   echo "480p fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-480p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
  if [ $VIDEOMODE -eq "129" ]; then
   echo "480i fullscreen!"
   sed "/^layout[ \t]/d" /home/zerogame/.wahcade/wahcade.ini > /home/zerogame/.wahcade/wahcade.tmp
   echo "layout                    $WAHCADETHEME-480p" >> /home/zerogame/.wahcade/wahcade.tmp
   mv /home/zerogame/.wahcade/wahcade.tmp /home/zerogame/.wahcade/wahcade.ini
  fi
fi
# Another one of those nice things thrown into this script is this very simple thing here.
# It will check to see if the MAME game list has been created and if not, creates it
# using a script to do it manually. This was done to avoid the appearance of Wahcade
# crashing when first indexing the users games giving the user no feedback.
# It still does take a long time, but at least the user can see that something's being done
# and knows that nothing's crashed. Other consoles don't have this problem in general
# since they make the game list in a pretty simple manner and don't bother with extra
# info. The side effect of this is that all the extra info (clones, date, etc...) doesn't make
# it into the file. I don't personally care since my themes don't use any of that info.
# Perhaps fix this before we go final, hmm?

if [ ! -f /home/zerogame/.wahcade/files/mame-0.lst ]; then
 if [ $USEZEROGAMEMAMELISTING -eq "1" ]; then
  xterm -e /home/zerogame/.zerogame/tools/createmamelist.sh
 fi
fi

# Run Gizmo Daemon in the background.
if [ $GIZMODENABLE -eq "1" ]; then
   gizmod &
fi

# Run Wahcade.
/usr/local/bin/wahcade

# As at the start of this script, we want to absolutely make sure that Gizmo is no longer
# running after Wahcade quits or dies for any reason. Kill it DEAD.
killall -9 gizmod

# Send a command to fluxbox to quit after Wahcade quits. This allows us to automatically
# restart in case Wahcade dies.
if [ $ZEROGAMEDEBUG -eq "0" ]; then 
  fluxbox-remote exit
fi

